/*----------CDAT Editor-----------------*
|  Display CDAT records, allow changes  |
*--------------------------------------*/

//  11/25/04 - initial version
//  3/8/07   - various fixes
//  3/27/07  - add create file, edit params
//  3/28/07  - add table displays
//  6/16/07  - block delete added

show_line "CEDIT.S - CDAT File Editor"
show_nl
noexit

#---inits---
init_var MAX_VARS = 19000
init_var NMAX 10000
$data_file = ""   // input file
$out_file = ""    // output file
$header = ""
$Delim = comma
N = 0
M = 0
Record = 1        // default record
Attr = 1          // default attribute
Params = 0        // parameters
NoPrompt = 0      // no prompts
Change = 0        // file has changed

call read_file:

#---action loop---
for_i = 1 1000
  show_nl
  show_line " File ops: (r)ead, (w)rite, (c)reate new file, (e)dit parameters"
  show_line " Record ops:  (d)isplay, (E)dit, (A)dd new, (D)elete, (M)ove"
  show_line " Attribute ops: (al)=list, (ae)=edit, (aa)=add, (ad)=delete, (am)=move"
  show_field " Miscl: Display (t/T)able, (h)eader, (W)rite tabular ..........(x)it->"
  input0 $action
  show_nl

  if ~M&~($action="r")&~($action="c")
    show_line "No fields defined, set up file now!"
    call create_file:
  end_if

  if $action="x"|$action="q"
    if Change
      query "-->File has changed, still exit?" y 
      if qq  exit
    else
      exit
    end_if
  end_if

  if $action="h"
    call show_header:
  end_if

  if $action="r"
    call read_file:
  end_if

  if $action="w"
    call write_file:
  end_if

  if $action="d"
    call disp_rec:
  end_if

  if $action="E"
    call edit_rec:
  end_if

  if $action="A"
    call add_rec:
  end_if

  if $action="D"
    call del_rec:
  end_if

  if $action="M"
    call move_rec:
  end_if

  if $action="al"
    call list_attr:
  end_if

  if $action="c"
    call create_file:
  end_if

  if $action="e"
    call edit_params:
  end_if

  if $action="t"
    call table:
  end_if

  if $action="T"
    call table2:
  end_if

  if $action="W"
    call write_tab:
  end_if

end_i

end


/*-----------------
  Create File
-----------------*/

create_file:
  input $data_file $header
  show_field "Number of fields in a record"
  input0 M
  for_i = 1 M
    input $Label[i]
  end_i
  show_line "Use (A)dd record to enter data, (w)rite to save"
  N = 0
  Record = 1
return


/*-----------------
  Show Header
-----------------*/

show_header:
  show_line "File: " $data_file ": " $header
  show_line "--> " @N " points in table"
  show_line "--> " @abs(M) " fields per point"
  show_line "--> " @Params " parameters"
  for_i = 1 Params       // read params, if any
    show_line "      param " @i ": " $plab[i] " = " $Par[i]
  end_i
return


/*-----------------------------
  read CDAT file...
------------------------------*/

/*-----------------------------
    Header line in    $header
    ne, na  in        N M
    # parameters      Params
      param lab         $plab[]
      param value       $Par[]
    labels            $Label[]
    data              $X[i][j]
------------------------------*/

read_file:
  if Change
    query "-->File has changed, still re-read?" y 
    if ~qq    return
  end_if

  #---prompt for CDAT file name---

  show_field "CDAT file name"
  input0 $data_file 

  #---read CDAT file--
  show_nl
  read_line $data_file $header
  if err return

  read_line $data_file N M    // rows/columns

  N = abs(N)
  if M=0 
    show_line "Number of fields=0, use (c)reate to set up file"
  else_if N=0  
    show_line "***Zero points in table, use (A)dd record to enter data***" \n
  end_if
  if N=1  N = NMAX

  if M<0                   // check for params
    M = -M
    read_line $data_file Params
    #---display param info---
    for_i = 1 Params       // read params, if any
      read_line $data_file $plab[i]
      read_line $data_file $Par[i]
    end_i
    show_nl
  else
    Params = 0
  end_if

  call show_header:

  #---read labels---
  set_delim space   // this drops the expl fields
  for_i = 1 M
    read_line $data_file $Label[i]
  end_i
  set_delim $Delim

  #---read data---
  show_field "reading..."
  for_i = 1 N
    if i*M>MAX_VARS
      show_line "file too large, first " @(i-1) " records read"
      N = i-1
      return
    end_if
    for_j = 1 M
      read_line $data_file $X[i][j]
      if eof
         N = i-1
         show_line "--> " @N " records in file"
        goto out:
      end_if
    end_j
  end_i
  show_line "done"

  out:
  show_nl
  close_file $data_file
  Change = 0
return


/*-------------------
    Write CDAT file
--------------------*/

write_file:
  str_len $out_file
  if S_len=0
    $out_file = $data_file
  end_if

  show_field "Output file name = "
  input0 $out_file

  show_nl
  #---write header---
  write_line $out_file $header
  show_line $header
  show_line "..." @N " points   " @M " fields   " @Params " params"
  show_nl

  if Params
    write_line $out_file @N " " @-M
  else
    write_line $out_file @N " " @M
  end_if

  #---write params---
  if Params
    write_line $out_file @Params
    for_i = 1 Params
      write_line $out_file $plab[i]
      write_line $out_file $Par[i]
    end_i
  end_if

  #---write_labels---
  if M
    for_i = 1 M
      write_line $out_file $Label[i]
    end_i

    #---write data---
    if N
      for_i = 1 N
        for_j = 1 M
          write_line $out_file $X[i][j]
        end_j
      end_i
    end_if
  end_if

  show_line "File " $out_file " written" \n
  close_file $out_file
  Change = 0
return


/*----------------
  Display Record
-----------------*/

disp_rec:
  if ~NoPrompt
    show_field "Record to Display="
    input0 Record
    if err return
  end_if

  if Record>N|Record<1
    show_line "Record " @Record " not in file"
    return
  end_if

  show_nl
  for_i = 1 M..
    show_line "  " $Label[i] "=" $X[Record][i]
  end_i
  if ~NoPrompt
    Record = Record+1
    if Record>N   Record = 1
  end_if
  show_nl
  NoPrompt = 0
return


/*-----------------
  Display Attribute
-----------------*/

list_attr:
  show_field "Attribute to Display="
  input0 Attr
  if err return

  if Attr>abs(M)|Attr<1
    show_line "Sorry, only " @abs(M) " attributes in file"
    return
  end_if

  show_nl
  show_line "Attribute = " $Label[Attr]

  for_i = 1 N
    show_line "  " @i "=" $X[i][Attr]
    if ~(i%40)
      query "CR to Continue..."
    end_if
  end_i
  show_nl
  Attr = Attr+1
  if Attr>M  Attr = 1
return


/*----------------
  Edit Record
-----------------*/

edit_rec:
  if ~NoPrompt
    show_field "Edit Record="
    input0 Record
    if err  return
    #---new record case---
    if Record>N|Record<1
      show_line "Record " @Record " doesn't exist, use (A)dd record"
      return
    end_if
  end_if

  show_nl
  for_i = 1 M..
    show_field "  " $Label[i] "=" 
    input0 $X[Record][i]
  end_i

  NoPrompt = 1
  call disp_rec:

  if ~NoPrompt
    Record = Record+1
    if Record>N  Record = 1
    show_nl
  end_if
  Change = 1
return


/*----------------
  Edit Params
-----------------*/

edit_params:
  Params0 = Params
  show_line "There are currently " @Params " parameters"
  show_field "  New number desired"
  input0 Params
  for_i = 1 Params
    if i>Params0  input $plab[i]
    show_field $plab[i] " = "
    input0 $Par[i]
  end_i
return

NoPrompt


/*----------------
  Add Record
-----------------*/

add_rec:
  Rec = N+1

  show_field "Add/Dup Record= (0 to abort)"
  input0 Rec
  if err return

  if Rec>N+1|Rec<1
    show_line "Bad record number, try again..."
    return
  end_if

  #---add record---
  N = N+1

  #---init record---
  for_i = 1 M
    $X[N][i] = ""
  end_i

  #---make room---
  show_nl
  if ~(Rec=N)
    for_i = N-1 Rec -1
      From = i
      To = i+1
      call copy_rec:
    end_i
    show_line "Record " @Rec " duplicated, input record " @(Rec+1)
    Rec = Rec+1
  else
    show_line "Record " @N " added, input record " @N
  end_if

  #---new record---
  Record = Rec
  NoPrompt = 1
  call edit_rec:

  NoPrompt = 0
  Change = 1
return


/*----------------
  Delete Record
-----------------*/

del_rec:
  show_field "Delete Record="
  input0 Rec
  if err return

  show_field "....to Record="
  input0 Rec2
  if err return


  if Rec>N|Rec<1|Rec2<Rec
    show_line "Record error: " @Rec "-" @Rec2
    return
  end_if

  #---move up---
  if ~(Rec=N)
    for_i = Rec2+1 N
      From = i
      To = i-(Rec2-Rec+1)
      call copy_rec:
    end_i
  end_if

#---erase last records---
  for_i = N-(Rec2-Rec) N
    for_j = 1 M
      $X[i][j] = ""
    end_j
  end_i

  show_line \n "Records " @Rec "-" @Rec2 " removed" \n

  N = N-(Rec2-Rec+1)

  Change = 1
return


/*----------------
  Move Record
-----------------*/

move_rec:
  show_field "Move Record "
  input0 Rec1
  if err return
  show_field "To "
  input0 Rec2
  if err return

  if Rec1>N|Rec1<1|Rec2>N|Rec2<1|Rec1=Rec2
    show_line \n "***Bad record number***" \n
    return
  end_if

  #---store record---
  for_i = 1 M
    $Xtmp[i] = $X[Rec1][i]
  end_i

  #---move block---
  if Rec1<Rec2
    for_i = Rec1+1 Rec2
      From = i
      To = i-1
      call copy_rec:
    end_i
  else
    for_i = Rec1-1 Rec2 -1
      From = i
      To = i+1
      call copy_rec:
    end_i
  end_if

  #---restore rec---
  for_i = 1 M
    $X[Rec2][i] = $Xtmp[i]
  end_i

  show_nl
  Change = 1
return


/*------------------
  copy rec utility
------------------*/

copy_rec:
  if To>N|To<1|From>N|From<1
    exit "Range error in copy_rec"
  end_if

  for_i = 1 M
    $X[To][i] = $X[From][i]
  end_i
return


/*---------------
   SHOW TABLE
----------------*/

table:
  if ~N*M 
    show_line "No Data"
    return
  end_if

  #---set ranges---
  init_var Nfirst = 1
  init_var Nlast = min(N,25)
  $Nrange = @Nfirst - @Nlast
  if N>25|~(Nfirst=1)
    show_field "table has " @N " records, range to display"
    input0 $Nrange
    str_sub $Nrange "-"
    str_tofloat Nfirst $S_head
    str_tofloat Nlast $S_tail
  else
    Nfirst = 1
    Nlast = N
  end_if

  init_var Mfirst = 1
  init_var Mlast = min(M,10)
  $Mrange = @Mfirst - @Mlast
  if M>10|~(Mfirst=1)
    show_field "table has " @M " fields, range to display"
    input0 $Mrange
    str_sub $Mrange "-"
    str_tofloat Mfirst $S_head
    str_tofloat Mlast $S_tail
  else
    Mfirst = 1
    Mlast = M
  end_if

  #---find col widths---
  Wid = 4
  for_i = Mfirst Mlast
    Col_wid[i] = 0
    str_len $Label[i]
    Col_wid[i] = max(Col_wid[i],S_len+1)
    for_j = Nfirst Nlast
      str_len $X[j][i]
      Col_wid[i] = max(Col_wid[i],S_len+1)
    end_j
    Wid = Wid+Col_wid[i]
    if Wid>80
      Mlast = i-1
      break
    end_if
  end_i

  #---label row---
  show_field "# | "
  for_i = Mfirst Mlast
    show_field $Label[i]
    str_len $Label[i]
    for_j = 1 Col_wid[i]-S_len
      show_field " "
    end_j
  end_i

  show_nl
  show_field "----"
  for_i = Mfirst Mlast
    for_j = 1 Col_wid[i]
      show_field "-"
    end_j
  end_i

  #---data---
  show_nl
  for_i = Nfirst Nlast
    show_field @i
    if i<10
      show_field " | "
    else
      show_field "| "
    end_if
    for_j = Mfirst Mlast
      show_field $X[i][j]
      str_len $X[i][j]
      for_k = 1 Col_wid[j]-S_len
        show_field " "
      end_k
    end_j
    show_nl
  end_i

  show_field "----"
  for_i = Mfirst Mlast
    for_j = 1 Col_wid[i]
      show_field "-"
    end_j
  end_i
  show_nl
return


/*---------------
   SHOW TABLE2
   (transpose)
----------------*/

table2:
  if ~N*M 
    show_line "No Data"
    return
  end_if

 #---set ranges---
  init_var Nfirst = 1
  init_var Nlast = min(N,10)
  $Nrange = @Nfirst - @Nlast
  if N>10|~(Nfirst=1)
    show_field "table has " @N " records, range to display"
    input0 $Nrange
    str_sub $Nrange "-"
    str_tofloat Nfirst $S_head
    str_tofloat Nlast $S_tail
  else
    Nfirst = 1
    Nlast = N
  end_if

  init_var Mfirst = 1
  init_var Mlast = min(M,25)
  $Mrange = @Mfirst - @Mlast
  if M>25|~(Mfirst=1)
    show_field "table has " @M " fields, range to display"
    input0 $Mrange
    str_sub $Mrange "-"
    str_tofloat Mfirst $S_head
    str_tofloat Mlast $S_tail
  else
    Mfirst = 1
    Mlast = M
  end_if

  #---find col widths---
  Wid = 0
  Col_wid[0] = 0
  for_i = Mfirst Mlast
    str_len $Label[i]
    Col_wid[0] = max(Col_wid[0],S_len+2)
  end_i
  Wid = Wid+Col_wid[0]
  for_i = Nfirst Nlast
    Col_wid[i] = 0
    for_j = Mfirst Mlast
      str_len $X[i][j]
      Col_wid[i] = max(Col_wid[i],S_len+1)
    end_j
    Wid = Wid+Col_wid[i]
    if Wid>80
      Nlast = i-1
      break
    end_if
  end_i

  #---label row---
  for_i = 1 Col_wid[0]
    show_field " "
  end_i
  for_i = Nfirst Nlast
    show_field @i
    S_len = floor(log(i))+1
    for_j = 1 Col_wid[i]-S_len
      show_field " "
    end_j
  end_i

  show_nl
  for_i = 1 Col_wid[0]
    show_field "-"
  end_i
  for_i = Nfirst Nlast
    for_j = 1 Col_wid[i]
      show_field "-"
    end_j
  end_i

  #---data---
  show_nl
  for_i = Mfirst Mlast
    show_field $Label[i] ":"
    str_len $Label[i]
    for_j = 1 Col_wid[0]-S_len-1
      show_field " "
    end_j
    for_j = Nfirst Nlast
      show_field $X[j][i]
      str_len $X[j][i]
      for_k = 1 Col_wid[j]-S_len
        show_field " "
      end_k
    end_j
    show_nl
  end_i

  for_i = 1 Col_wid[0]
    show_field "-"
  end_i
  for_i = Nfirst Nlast
    for_j = 1 Col_wid[i]
      show_field "-"
    end_j
  end_i
  show_nl
return


/*-------------------------
   WRITE TABLULAR DATA FILE
---------------------------*/

write_tab:

  show_line "--> Write data in tabular format <---"
  query ".....continue?" y
  if ~qq  return

  #---name the reformatted file---
  show_nl
  str_len $out_file
  if S_len=0
    $out_file = $data_file
  end_if
  $dat_out = $out_file
  set_file_type $dat_out csv
  show_field "    Output file name"
  input0 $dat_out

  show_nl
  show_line  "    Supported delimiters: comma,tab,space,slash"
  show_field "    Specify delimiter" 
  input0 $Delim
  set_delim $Delim
  show_nl

  Mcol = abs(M)
  show_field "Number of columns to write?"
  input0 Mcol
  show_nl

  #---write the file---
  write_line $dat_out $header
  if err  return

  #---write parameters---
  for_i = 1 Params
    write_line $dat_out "  " $plab[i] " = " $Dlm $Par[i]
  end_i

  #---write labels---
  write_field $dat_out $Label[1]
  for_i = 2 Mcol           // labels
    write_field $dat_out $Dlm + $Label[i]
  end_i
  write_field $dat_out \n

  #---write data, allow alpha---
  for_i = 1 N
    write_field $dat_out $X[i][1]
    for_j = 2 Mcol
      str_len $X[i][j]
      if S_len=0
        write_field $dat_out $Dlm
      else
        write_field $dat_out $Dlm $X[i][j]
      end_if
    end_j
    write_field $dat_out \n
  end_i
  show_nl
  show_line "File " $dat_out " written"
  close_file $dat_out
return
