/*====TVM.s===*/
/*  Copyright Stellingwerf Consulting, 2007  */

show_line "*==============TIME VALUE of MONEY=============*"
show_line "|  Variables are...                            |"
show_line "|    PV = Present Value                        |"
show_line "|    FV = Future Value                         |"
show_line "|    Nyr = number of years                     |"
show_line "|    Iyr = Interest rate (%/yr)                |"
show_line "|    Pmt = Payment                             |"
show_line "|    Pyr = Number of payments per year         |"
show_line "|    Pch = Payment change (%/yr)               |"
show_line "|    Tax = Tax rate (%) (on interest only)     |"
show_line "|    Pmt_at_beg = 1 if pmt at beg of period    |"
show_line "|               = 0 if pmt at end              |"
show_line "|  Sign Convention:                            |"
show_line "|    DEPOSITS    < 0                           |"
show_line "|    WITHDRAWALS > 0                           |"
show_line "|  Type 'end' to terminate data input          |"
show_line "*==============================================*"

show_line \n "Default Data..."

#---HP auto loan example--- 
$Title = "HP Auto Loan Example"
Nyr = 4
PV = 18280
Iyr = 13
Pmt = -490.41
Pyr = 12
Pch = 0
FV = 0
Pmt_at_beg = 0
Tax = 0
$Choice = m

Calc = false
decimals 2
show_nl
show_line $Title
show PV FV Iyr
show Nyr Pmt Pyr
show Pmt_at_beg Pch Tax
show_nl

#---main calculation loop here----
for_i = 1 100
  call doit:
end_i

end

/*-----------------------------------------------------------*/

doit:
  call params:
  $Choice0 = $Choice    //  prev choice, used for "w"
  show_line  "Options:   (a)mort table, (l)ink FV<>PV, (r)ead,(w)rite file, (q)uit"
  show_field   "  Solve for (f)V (p)V (n)yr (i)nt P(m)t:"
  input0 $Choice
  show_nl
  if $Choice="q"|$Choice="x"
    exit
  else_if $Choice="r"
    show_line "File to read (.txt will be added)"
    input $File
    if ~$File  $File = test
    $File1 = $File + ".txt"
    read_file $File1
    show_nl
    show_line $Title
    show PV FV Iyr
    show Nyr Pmt Pyr
    show Pmt_at_beg Pch Tax
    show_nl
  else_if $Choice="w"
    show_line "File to write (.txt will be added)"
    input $File
    if ~$File  $File = test
    $File1 = $File + ".txt"
    input $Title
    $Choice = $Choice0
    write_var $File1 $Title Nyr PV Iyr Pmt Pyr Pch FV Pmt_at_beg Tax $Choice
    show_line $File1 " written"
    close_file $File1
    show_nl
  else_if $Choice="a"
    call amort:
    $Choice = $Choice0
  else_if $Choice="l"
    Tmp = PV
    PV = -FV
    FV = -Tmp
    show PV FV
    show_nl
    $Choice = $Choice0

  #---calculations-------------------------
  else_if $Choice="f"
    input PV Iyr Nyr Pmt Pyr Pmt_at_beg Pch Tax
    call params:
    if Alpha=Beta
      FV = -PV*Alpha^N-Pmt*N*Alpha^(N-1)*(1+Pmt_at_beg*Int)
    else
      FV = -PV*Alpha^N-Pmt*(Alpha^N-Beta^N)*(1+Pmt_at_beg*Int)/(Alpha-Beta)
    end_if
    show_nl
    show_field "  FV = "
    show_tab FV
    show_nl
  else_if $Choice="p"
    input FV Iyr Nyr Pmt Pyr Pmt_at_beg Pch Tax
    call params:
    if Alpha=Beta      
      PV = (Pmt*N*Alpha^(N-1)*(1+Pmt_at_beg*Int)-FV)/Alpha^N 
    else
      PV = (Pmt*(Alpha^N-Beta^N)*(1+Pmt_at_beg*Int)/(Alpha-Beta)-FV)/Alpha^N 
    end_if
    show_nl
    show_field "  PV = "
    show_tab PV
    show_nl
  else_if $Choice="n"
    input PV FV Iyr Pmt Pyr Pmt_at_beg Pch Tax
    call params:
    if Pmt
      N = abs((FV+PV)/Pmt)    // guess
    else
      N = abs((FV+PV)/(Int*PV))
    end_if
    //show_line "Guess = " @N
    Passes = 0
    for_i = 1 100
      call fin_function:    // compute F_s
      X_s = N
      call update_X_s: ../math/secant.s
      N = X_s
      //show j N F_s
      break_if Converge_s
    end_i
    if i=101
      Noconverge = true
      show_line "No convergence, adjust values"
      return
    end_if
    Nyr = N/Pyr
    show_nl
    show_field "  Nyr = "
    show_tab0 Nyr
    decimals 0
    show_tab0 i
    decimals 2
    show_line "  Iterations"
    show_nl
  else_if $Choice="m"
    input PV FV Iyr Nyr Pyr Pmt_at_beg Pch Tax
    call params:
    if Alpha=Beta
      Pmt = (-FV-PV*Alpha^N)/(N*Alpha^(N-1)*(1+Pmt_at_beg*Int))
    else
      Pmt = (-FV-PV*Alpha^N)/((Alpha^N-Beta^N)*(1+Pmt_at_beg*Int)/(Alpha-Beta))
    end_if
    show_nl
    show_field "  Pmt = "
    show_tab Pmt
    show_nl
  else_if $Choice="i"
    input PV FV Nyr Pmt Pyr Pmt_at_beg Pch Tax
    N = Nyr*Pyr
    Int = .10/Pyr    // guess
    Alpha = 1+Int
    Beta = 1+Pch/(Pyr*100)
    //show_line "Guess = " @Int
    Passes = 0
    for_i = 1 100
      Alpha = 1+Int
      call fin_function:    // compute F_s
      X_s = Int
      call update_X_s: ../math/secant.s
      Int = X_s
      //show j Int F_s
      break_if Converge_s
    end_i
    if i=101
      Noconverge = true
      show_line "No convergence, adjust values"
      return
    end_if
    Iyr = Int*(Pyr*100)/(1-Tax/(Pyr*100))
    show_nl
    show_field "  Iyr = "
    show_tab0 Iyr
    decimals 0
    show_tab0 i
    decimals 2
    show_line "  Iterations"
    show_nl
    Calc = true
  else 
    show_line "Invalid choice, try again..."
  end_if
return

#--------------------------------------------------------#

params:
  Int = (1-Tax/100)*Iyr/(Pyr*100)
  Alpha = 1+Int
  Beta = 1+Pch/(Pyr*100)
  N = Nyr*Pyr
  Calc = true
return

amort:
  if ~Calc
    show_line "***Amort: not all variable defined, do a <Solve> first***" \n
    return
  end_if
  Tot_int = 0
  Tot_prin = 0
  Tot_pmt = 0
  Tot_tax = 0
  Val = PV
  Pmti = Pmt
  Np = Pyr
  if Pyr=1 Np = 10
  N_amort = N
  show_field "Number of periods for table"
  input0 N_amort
  write_lab amort_table.csv "Pmt #" Payment= Interest- Tax+ Principal Balance
  show_lab0 "Pmt #" Payment= Interest
  if Tax  show_lab0 -Tax
  show_lab +Principal Balance
  decimals 0
  if Tax
    show_tab0 0 0 0 0 0
    decimals 2
    show_tab Val
  else
    show_tab0 0 0 0 0
    decimals 2
    show_tab Val
  end_if
  decimals 2
  for_i = 1 N_amort
    I = -Iyr/(Pyr*100)*Val
    Taxa = I*Tax/100
    Prin = Pmti-(I-Taxa)
    Tot_prin = Tot_prin+Prin
    Tot_int = Tot_int+I
    Tot_tax = Tot_tax+Taxa
    Tot_pmt = Tot_pmt+Pmti
    Val = Val-(I-Taxa)+Pmti
    decimals 0
    show_tab0 i 
    decimals 2
    show_tab0 Pmti I
    if Tax show_tab0 Taxa
    show_tab Prin Val
    write_dat i Pmti I Taxa Prin Val
    Pmti = Pmti*Beta
    if ~(i%Np)
      show Tot_pmt Tot_int Tot_prin
      if Tax  show Tot_tax
    end_if
  end_i
  if (i-1)%Np
    show Tot_pmt Tot_int Tot_prin
    if Tax show Tot_tax
  end_if
  show_nl
  close_file amort_table.csv
  show_line "   -->Data written to amort_table.csv<--"
  show_nl
return

/*---function to solve for Int, N -------------*/
fin_function:
    if Alpha=Beta
      F_s = FV+PV*Alpha^N+Pmt*N*Alpha^(N-1)*(1+Pmt_at_beg*Int)
    else
      F_s = FV+PV*Alpha^N+Pmt*(Alpha^N-Beta^N)*(1+Pmt_at_beg*Int)/(Alpha-Beta)
    end_if
return

