#---Power Fit---R. Stellingwerf---
#    4/13/2004 Version

#  fit a data set to a function of the form
#    F(x) = a * (x - b) ^ c

#   Divisor = 1/a
#   Shift = b
#   M_r = c
#   R2_r = R^2 of the fit = % of variance accounted for

show_line "POWER_FIT.S - shifted power law fit"
show_nl
Shift = 0
Enter_data = false

#---default data file---
$datf = zukas.csv         #<<< current problem

#---data entry if desired---
query "Enter Data?" Enter_data
if Enter_data
  show_nl;
  show_line "Enter Data, type 'done' when finished"
  show_nl;
  begin_loop_i = 1 400
    Ydat[i] = i
    input Xdat[i] Ydat[i]
    show_nl
    break_if eof
  end_loop_i
  N = i-1
  if N=0; exit "Zero points in table"; end_if

  show_field "    Data file name"
  input0   $datf    // change data file name?
  $datp = $datf

  write_lab $datf X Y
  begin_loop_i = 1 N
    write_dat Xdat[i] Ydat[i]
  end_loop_i
  show_line "File " $datf " written"
  close_file $datf
else
  show_field "    Data file name"
  input0   $datf    // change data file name?
  $datp = $datf

  read_lab $datf
  begin_loop_i = 1 400
    read_dat Xdat[i] Ydat[i]
    break_if eof
  end_loop_i
  if i=401
      exit "Max points read, probable eof error"
  end_if

  N = i-1
  NN = Ydat[N]
  if N<2; exit "Need >1 points in table"; end_if
  show_line "File " $datf " read"
end_if

 #---option to see data---
query "Review Data?" n
if qq
  show_nl;  show_field "        ";  show Shift N
  show_nl;  show_lab X Num
  begin_loop_i = 1 N
    show_tab Xdat[i] Ydat[i]
    if ~(i%25); query Continue..(CR)..; end_if
  end_i
  show_nl
  query "cr to analyze..."
end_if
show_nl

#---regress loop------
Tol_g = 1.e-3
X0_g = Xdat[1]

do_i = 1 100

  call p_fit:      //---> main calc call

  Shift_old = Shift
  X_g = Shift;  F_g = R2_r;
  call update_X_g: extremum.s    //---> get next Shift
  Shift = X_g;

  if i>5            //---> smooth out oscillations
    Shift = (Shift+Shift_old)/2
    //show_line "---> smooth oscillations"
  end_if

  break_if Converge_g
end_i

show_nl; show_line "    Finished, fit params in pfit.inp"
show_nl; show M_r Divisor Shift R2_r; show_nl
write_var pfit.inp $datp M_r Divisor Shift R2_r N NN Tol_g

end

#------------define the fitting formula---------------------------

p_fit:

#---data for regression---
begin_loop_i = 1 N
  if Ydat[i]=0
    exit "Y = 0"
  end_if
  if (Xdat[i]-Shift)=0
    Shift = 0.90*Shift
    show_line "---> shift equals x[" @i "], corrected"
  end_if
  X_r[i] = log(Xdat[i]-Shift)
  Y_r[i] = log(Ydat[i])
end_loop_i

#---regression parameters---
N_r = N
Enter_data = true     // regress data already set
$datf = p2dat.csv     // regress will write to this file
Interactive = false  
Write_fit = true      // fit will be in regr.csv

call regress: regress.s     // REGRESS

Divisor = 10^(-B_r)
show M_r Divisor Shift R2_r

return

