#---cubic equation solver---
# 11/2003 - rfs

#  find roots of a cubic equation
#  ax^3 + bx^2 + cx + d = 0

#  NR iteration to real root:
#  xnew = (2ax^3 + bx^2 - d) / (3ax^2 + 2bx + c)

#  if r1 is the real root, and q = (b/a + r1)
#    then the other two roots are given by:
#  r2,3 = -q/2 +/- sqrt( (q/2)^2 - (q*r1 + c/a) )

#----define the default problem-----

do_cubic:

A = 1;  B = -6;  C = 11;  D = -6
//A = 1;  B = 3;  C = 3;  D = 1
//A = 1;  B = -1;  C = 1;  D = -1

#---header---
show_line "S coded cubic solver - type Ctrl-C to exit"
show_nl

#---main calling loop---
begin_loop_i = 1 10
    call cubic:
end_loop_i
end

#----------------------------------------------------
cubic:

show_nl
show_line "Define cubic equation:"
show_line "   A x^3 + B x^2 + C x + D = 0"; show_nl

input A B C D

call show_cubic:      //  display routine

Cs = 0
call check_special:   //  special cases routine
if Cs; return; end_if

X1 = 1.1    //  guess

#---find a good guess---
begin_loop_i = 1 5
    break_if 3*A*sq(X1)+2*B*X1+C
    X1 = X1+0.1
end_loop_i

#---iterate to first root---
begin_loop_i = 1 100
    Denom = 3*A*sq(X1)+2*B*X1+C
    break_if abs(Denom)<1.e-15
    X2 = (2*A*cub(X1)+B*sq(X1)-D)/Denom
    break_if ~(abs(X2)+abs(X1))
    Delta = abs((X2-X1))/(abs(X2)+abs(X1))
    //show i X2 Delta    //  debug
    X1 = X2
    break_if Delta<1.e-8
end_loop_i

show_line "Real Root... " @i " iterations";  show_nl

show X1; show_nl

#---compute other two roots---
Q = B/A+X1
Dis = sq(Q/2)-(Q*X1+C/A)

if Dis>0|Dis=0
    X2 = -Q/2-sqrt(Dis)
    X3 = -Q/2+sqrt(Dis)
    show X2;    show_nl;    show X3
else
    Re = -Q/2
    Im = sqrt(-Dis)
    show_line "---Complex Roots---"; show_nl
    show_line "X2 = ( " @Re ", +" @Im " )"; show_nl
    show_line "X3 = ( " @Re ", -" @Im " )"
end_if  
show_nl

return
#---------------------------------------------------#

#---pretty print the cubic---
  
show_cubic:

show_nl;  show_field "==> Cubic:  "

if A=1;        show_field "x^3"
else_if A=-1;  show_field "-x^3"
else_if A;     show_field @A " x^3"
end_if

if B=1;        show_field " + x^2"
else_if B=-1;  show_field " - x^2"
else_if B>0;   show_field " + " @B " x^2"
else_if B;     show_field " - " @-B " x^2"
end_if

if C=1;        show_field " + x"
else_if C=-1;  show_field " - x"
else_if C>0;   show_field " + " @C " x"
else_if C;     show_field " - " @-C " x"
end_if

if D>0;        show_field " + " @D
else_if D;     show_field " - " @-D
end_if

show_line " = 0"; show_nl
return
#---------------------------------------------------#

#---check for special cases---

check_special:

if ~A&~B&~C;  //  too many zeros!
    show_line "No solution!"
    Cs = 1;  return
end_if

if ~A&~B     //  linear case
    show_line "X1 = " @-D/C; show_nl
    show_line "X2 = X3 = no solutions (linear case)"
    Cs = 1;  return
end_if

if ~A&B      // quadratic case
    Q = sq(C)-4*B*D
    if Q>0|Q=0
        show_line "X1 = " @(-C+sqrt(Q))/2; show_nl
        show_line "X2 = " @(-C-sqrt(Q))/2; show_nl
    else
        show_line "---Complex Roots---"; show_nl
        Re = -C/2
        Im = sqrt(-Q)/2
        show_line "X1 = (" @Re ", +" @Im ); show_nl
        show_line "X2 = (" @Re ", -" @Im ); show_nl
    end_if
    show_line "X3 = no solution (quadratic case)"
    Cs = 1;  return
end_if

return
#---------------------------------------------------#