#----REGRESSION PACKAGE - R. Stellingwerf----
#    1/20/2004 Version - _r variable names

show_line "REGRESS.S - S Coded Regression"
show_nl
Entry = false

#---default data file---
$datf = reg.csv

#---data entry if desired---
query "Enter Data?" n
Enter_data = qq
if Enter_data
  show_nl;
  show_line "Enter Data, type 'done' when finished"
  show_nl;
  begin_loop_i = 1 400
    input X_r[i] Y_r[i]
    break_if eof
  end_loop_i
  N_r = i-1
  if N_r=0; exit "Zero points in table"; end_if
end_if

show_field "    Data file name"
input0   $datf    // change data file name?

#---regress loop------

do_i 1 100
    call regress:
end_i

end

#----end of interactive setup-------------------------

#---use "call regress:" to call from another package
#  define file $datf (default = reg.csv)
#  then if Enter_data is true
#     X_r[] Y_r[] and N_r must be set
#     and this data set will be written to $datf
#  if Enter_data is false
#     the data will be read from $datf
#  set Interactive = false for no queries or display
#    in this case also set:
#        LogX_r LogY_r  Scatter_x
#        {Fixed_point X0_r Y0_r}
#        {Write_fit $fitf}
#  answer is in M_r and B_r  (slope and intercept)
#    uncertainties are in SigM and SigB

regress:

#---entry inits if not set---
init_var Entry = true
init_var Enter_data = false
init_var Write_fit = true
init_var Interactive = true
init_var $datf = reg.csv
init_var $fitf = regr.csv
init_var Fixed_point = false
init_var X0_r = 0
init_var Y0_r = 0
init_var LogX_r = false
init_var LogY_r = false
init_var Scatter_x = false
init_var $Xlab_r = X
init_var $Ylab_r = Y

if Interactive
  show_nl; show_line "----Entering Regression Package----";show_nl;
end_if

show $Xlab_r $Ylab_r

#---read or store data---
if Enter_data
  write_lab $datf $Xlab_r $Ylab_r
  begin_loop_i = 1 N_r
    write_dat X_r[i] Y_r[i]
  end_loop_i
  if Interactive; show_line "File " @$datf " written"
  end_if
else
  read_lab $datf $Xlab_r $Ylab_r
  begin_loop_i = 1 400
    read_dat X_r[i] Y_r[i]
    break_if eof
  end_loop_i
  if i=401
      exit "Max points read, probable eof error"
  end_if

  N_r = i-1
  if N_r=0; exit "Zero points in table"; end_if
  if Interactive; show_line "File " @$datf " read"
  end_if
end_if

show $Xlab_r $Ylab_r

if Interactive
  show_line "--> " @N_r " points in table"
  show_nl

  #---option to see data---
  query "Review Data?" n
  if qq
    show_nl; show_lab $Xlab_r $Ylab_r
    begin_loop_i = 1 N_r
      show_tab X_r[i] Y_r[i]
      if ~(i%25); query Continue..(CR)..; end_if
    end_i
    show_nl
  end_if

  #---option for fixed point---
  query "Specify Fixed Point for Fit?" Fixed_point
  Fixed_point = qq
  if qq
    input X0_r Y0_r
  else
    X0_r = 0; Y0_r = 0
  end_if

  #---options for log or exp fit---
  query "Log X_r (log fit)?" LogX_r
  if LogX_r&Fixed_point&~X0_r
    show_nl
    show_line "    !!!Cannot have X0=0 with LogX, try again!!!"
    show_nl
    return
  end_if

  if LogX_r
    query "Log Y (power law fit)?" LogY_r
  else
    query "Log Y (exp fit)?" LogY_r
  end_if

  if LogY_r&Fixed_point&~Y0_r
    show_nl
    show_line "    !!!Cannot have Y0=0 with LogY, try again!!!"
    show_nl
    return
  end_if
  show_nl
end_if

#---begin calculation-------
Sx = 0; Sy = 0; Sxy = 0
Sx2 = 0; Sy2 = 0
begin_loop_i = 1 N_r
  if LogX_r
    if X_r[i]>0
      X$ = log(X_r[i])
      if Fixed_point&X0_r>0; X$ = X$-log(X0_r); end_if
    else; continue; end_if
  else; X$ = X_r[i]-X0_r; end_if
  if LogY_r
    if Y_r[i]>0
      Y$ = log(Y_r[i])
      if Fixed_point&Y0_r>0; Y$ = Y$-log(Y0_r); end_if
    else; continue; end_if
  else; Y$ = Y_r[i]-Y0_r; end_if
  Sx = Sx+X$
  Sy = Sy+Y$
  Sxy = Sxy+X$*Y$
  Sx2 = Sx2+X$*X$
  Sy2 = Sy2+Y$*Y$
end_loop_i

Sigx2 = (Sx2-Sx*Sx/N_r)/(N_r-1)
Sigy2 = (Sy2-Sy*Sy/N_r)/(N_r-1)
Sigxy = (Sxy-Sx*Sy/N_r)/(N_r-1)

if Fixed_point
  if LogX_r; Xx0 = log(X0_r)
  else;    Xx0 = X0_r
  end_if
  if LogY_r; Yy0 = log(Y0_r)
  else;    Yy0 = Y0_r
  end_if

  M_r = Sxy/Sx2
  B_r = Yy0-M_r*Xx0
else
  M_r = Sigxy/Sigx2
  B_r = (Sy-M_r*Sx)/N_r
end_if

#---double scatter case---
if Sigx2>0
  Mxy = sign(Sigxy)*sqrt(Sigy2/Sigx2)
else
  Mxy = 1
end_if
Bxy = (Sy-Mxy*Sx)/N_r

Tmp = Sigx2*Sigy2
if Tmp>0
  R2_r = sq(Sigxy)/Tmp
else
  R2_r = sign(Sigxy)
end_if

SigB = sqrt((N_r-1)*Sigy2*(1-R2_r)/(N_r-2))
SigM = SigB*sqrt(1/(Sigx2*(N_r-1)))

#---repeat to estimate error bars---
Sx = 0; Sy = 0; Sxy = 0
Sx2 = 0; Sy2 = 0; Ey2 = 0
begin_loop_i = 1 N_r
  X$ = X_r[i];  Y$ = Y_r[i]
  if ~LogX_r&~LogY_r
    Fit = M_r*X$+B_r
  else_if LogX_r&LogY_r
    Fit = X$^M_r*exp10(B_r)
  else_if LogX_r
    if X$>0
      Fit = M_r*log(X$)+B_r
    else; continue
    end_if
  else_if LogY_r
    Fit = exp10(M_r*X$+B_r)
  end_if

  Sy = Sy+Y$
  Sy2 = Sy2+Y$*Y$
  Ey2 = Ey2+sq(Y$-Fit)
end_loop_i

Sigy2 = (Sy2-Sy*Sy/N_r)/(N_r-1)
Sigey2 = Ey2/(N_r-1)

R2_fit = 1-Sigey2/Sigy2

#---show answer---
if Interactive
  show_line "Regression results:"
  if ~LogX_r&~LogY_r; show_line "    Y = M*X + B"
  else_if LogX_r&LogY_r; show_line "    Y = X^M * 10^B"
  else_if LogX_r; show_line "    Y = M*log(X) + B"
  else_if LogY_r; show_line "    Y = 10^( M*X + B )"
  end_if
  show_nl
  show_line "  M = " @M_r " (+/-" @SigM ")" "   B = " @B_r " (+/-" @SigB ")"
  show_line "  R^2 = " @R2_r "  % Variance modeled = " @R2_fit*100
end_if

#---option to do double scatter case---
if Interactive&~Fixed_point
  show_nl
  query "Scatter in both X and Y?" Scatter_x
end_if

if Fixed_point; Scatter_x = false; end_if

if Scatter_x&Interactive
  show_nl
  show_line "XYdev results: ";show_nl
  show_line "Slope(xy) = " @Mxy "    Intercept(xy) = " @Bxy
  M_r = Mxy; B_r = Bxy
end_if

#---option to plot the fitting line---
if Interactive
  show_nl
  query "Write plot file?" Write_fit
end_if

if Write_fit
  if Interactive
    show_field "    Plot file name"
    input0 $fitf
  end_if
  write_lab $fitf $Xlab_r $Ylab_r Fit
  begin_loop_i = 1 N_r
    X$ = X_r[i];  Y$ = Y_r[i]
    if ~LogX_r&~LogY_r
      Fit = M_r*X$+B_r
    else_if LogX_r&LogY_r
      Fit = X$^M_r*exp10(B_r)
    else_if LogX_r
      if X$>0
        Fit = M_r*log(X$)+B_r
      else; continue
      end_if
    else_if LogY_r
      Fit = exp10(M_r*X$+B_r)
    end_if
    write_dat X$ Y$ Fit
  end_loop_i
  if Interactive
    show_nl
    show_line "Data with fit written to " $fitf
  end_if
  close_file $fitf
end_if
return
